"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLoader = exports.createUseLoader = void 0;
var tslib_1 = require("tslib");
var aggregateToQuery_1 = require("./aggregateToQuery");
var RTKLoader_1 = require("./RTKLoader");
var mergeConfig = function (original, extended) {
    if (!original)
        return extended;
    return tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, original), extended), { deferred: tslib_1.__assign(tslib_1.__assign({}, original.deferred), extended.deferred) });
};
var createUseLoader = function (createUseLoaderArgs) {
    var useLoader = function () {
        var _a, _b, _c;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var loaderRes = createUseLoaderArgs.useQueries.apply(createUseLoaderArgs, args);
        var queriesList = loaderRes.queries
            ? Object.keys(loaderRes.queries).map(function (key) { return loaderRes.queries[key]; })
            : [];
        var aggregatedQuery = (0, aggregateToQuery_1.aggregateToQuery)(queriesList);
        if ((_b = (_a = createUseLoaderArgs.config) === null || _a === void 0 ? void 0 : _a.deferred) === null || _b === void 0 ? void 0 : _b.shouldThrowError) {
            if (loaderRes.deferredQueries) {
                var deferredQueriesList = Object.keys(loaderRes.deferredQueries).map(function (key) { return loaderRes.deferredQueries[key]; });
                if (deferredQueriesList.some(function (q) { return q.isError; })) {
                    aggregatedQuery.isSuccess = false;
                    aggregatedQuery.isError = true;
                    aggregatedQuery.error = (_c = deferredQueriesList.find(function (q) { return q.isError; })) === null || _c === void 0 ? void 0 : _c.error;
                }
            }
        }
        if (aggregatedQuery.isSuccess || queriesList.length === 0) {
            var data = createUseLoaderArgs.transform
                ? createUseLoaderArgs.transform(loaderRes)
                : loaderRes;
            return tslib_1.__assign(tslib_1.__assign({}, aggregatedQuery), { isSuccess: true, data: data, currentData: data, originalArgs: args });
        }
        return aggregatedQuery;
    };
    useLoader.original_args = createUseLoaderArgs;
    return useLoader;
};
exports.createUseLoader = createUseLoader;
/**
 * Creates a `loader` that can be used to fetch data and render error & loading states.
 * @example
 * const loader = createLoader({
 *  queriesArg: (props) => props.userId,
 *  useQueries: (userId) => {
 *    const user = useGetUserQuery(userId);
 *    return { queries: { user } };
 *  },
 *  onError: (error) => <ErrorView error={error} />,
 *  onLoading: () => <LoadingView />,
 * });
 */
var createLoader = function (createLoaderArgs) {
    var _a, _b, _c, _d;
    var useLoader = (0, exports.createUseLoader)({
        useQueries: (_a = createLoaderArgs.useQueries) !== null && _a !== void 0 ? _a : (function () { return ({}); }),
        transform: createLoaderArgs.transform,
        config: createLoaderArgs.config,
    });
    var loader = {
        useLoader: useLoader,
        onLoading: createLoaderArgs.onLoading,
        onError: createLoaderArgs.onError,
        onFetching: createLoaderArgs.onFetching,
        whileFetching: createLoaderArgs.whileFetching,
        queriesArg: createLoaderArgs.queriesArg,
        config: createLoaderArgs.config,
        LoaderComponent: (_d = (_c = (_b = createLoaderArgs.config) === null || _b === void 0 ? void 0 : _b.loaderComponent) !== null && _c !== void 0 ? _c : createLoaderArgs.loaderComponent) !== null && _d !== void 0 ? _d : RTKLoader_1.RTKLoader,
        extend: function (_a) {
            var _b, _c;
            var useQueries = _a.useQueries, transform = _a.transform, extendedArgs = tslib_1.__rest(_a, ["useQueries", "transform"]);
            var original = this;
            var mergedConfig = mergeConfig((_b = original.config) !== null && _b !== void 0 ? _b : {}, (_c = extendedArgs.config) !== null && _c !== void 0 ? _c : {});
            backwardsSupportLoaderComponent(mergedConfig, extendedArgs);
            var extendedLoader = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, original), extendedArgs), { config: mergedConfig });
            if (useQueries) {
                var newUseLoader = (0, exports.createUseLoader)({
                    useQueries: useQueries,
                    transform: transform,
                    config: mergedConfig,
                });
                extendedLoader.useLoader = newUseLoader;
            }
            else if (transform) {
                var newUseLoader = (0, exports.createUseLoader)({
                    useQueries: extendedLoader.useLoader.original_args.useQueries,
                    transform: transform,
                    config: mergedConfig,
                });
                extendedLoader.useLoader = newUseLoader;
            }
            else if (extendedArgs.config) {
                var newUseLoader = (0, exports.createUseLoader)({
                    useQueries: extendedLoader.useLoader.original_args.useQueries,
                    transform: extendedLoader.useLoader.original_args.transform,
                    config: mergedConfig,
                });
                extendedLoader.useLoader = newUseLoader;
            }
            return extendedLoader;
        },
    };
    return loader;
};
exports.createLoader = createLoader;
function backwardsSupportLoaderComponent(mergedConfig, extendedArgs) {
    if (!mergedConfig.loaderComponent &&
        extendedArgs.loaderComponent) {
        mergedConfig.loaderComponent = extendedArgs.loaderComponent;
    }
    return mergedConfig;
}
//# sourceMappingURL=createLoader.js.map