import * as Types from "./types";
/**
 * A higher order component that wraps a component and provides it with a loader.
 * @param Component The component to wrap with a loader. Second argument is the resolved loader data.
 * @param loader The loader to use.
 * @returns A component that will load the data and pass it to the wrapped component.
 * @example
 * const Component = withLoader((props, loaderData) => {
 *  return <div>{loaderData.queries.user.name}</div>;
 * }, loader);
 */
export declare const withLoader: <TProps extends Record<string, any>, TReturn extends unknown, TQueries extends Record<string, Types.UseQueryResult<unknown>>, TDeferred extends Record<string, Types.UseQueryResult<unknown>>, TPayload extends unknown, TArg = never>(Component: Types.ComponentWithLoaderData<TProps, Types.Unwrap<TReturn>>, loader: Types.Loader<TProps, TReturn, TQueries, TDeferred, TPayload, TArg>) => Types.Component<TProps>;
