import * as Types from "./types";
export declare const createUseLoader: <TQueries extends Record<string, Types.UseQueryResult<unknown>>, TDeferred extends Record<string, Types.UseQueryResult<unknown>>, TPayload extends unknown, TReturn extends unknown = Types.ResolveDataShape<Types.MakeDataRequired<TQueries>, TDeferred, TPayload>, TArg = never>(createUseLoaderArgs: Types.CreateUseLoaderArgs<TQueries, TDeferred, TPayload, TReturn, TArg>) => Types.UseLoader<TArg, TReturn, TQueries, TDeferred, TPayload>;
/**
 * Creates a `loader` that can be used to fetch data and render error & loading states.
 * @example
 * const loader = createLoader({
 *  queriesArg: (props) => props.userId,
 *  useQueries: (userId) => {
 *    const user = useGetUserQuery(userId);
 *    return { queries: { user } };
 *  },
 *  onError: (error) => <ErrorView error={error} />,
 *  onLoading: () => <LoadingView />,
 * });
 */
export declare const createLoader: <TProps extends unknown, TQueries extends Record<string, Types.UseQueryResult<unknown>>, TDeferred extends Record<string, Types.UseQueryResult<unknown>>, TPayload extends unknown, TReturn extends unknown = Types.ResolveDataShape<Types.MakeDataRequired<TQueries>, TDeferred, TPayload>, TArg extends unknown = never>(createLoaderArgs: Types.CreateLoaderArgs<TProps, TQueries, TDeferred, TPayload, TReturn, TArg>) => Types.Loader<TProps, TReturn, TQueries, TDeferred, TPayload, TArg>;
